@@ -56,9 +56,7 @@ import parse_requirements
 from find_module import find_python_module
 from engage_django_components import validate_django_components
 from package_data import PREINSTALLED_PACKAGES, get_apps_for_packages, \
-     extend_app_list, PACKAGES_FOR_KNOWN_APPS
-
-verbose = False
+     PACKAGES_FOR_KNOWN_APPS
 
 
 SETTINGS_FILENAME = "settings.py"
@@ -69,6 +67,7 @@ def run_safe_validations(handler):
     """This function runs all the validations that can safely be done without running
     user code. It returns the common parent directory of the archive.
     """
+    logger.info(">> Running safe validations")
     # check archive format
     namelist = handler.get_namelist()
     common_dir = validate_archive_files(namelist)
@@ -118,7 +117,7 @@ def check_url_setting(setting_name, setting_value, results, default_value='',
         if path_should_end_in_slash and (sr[2][len(sr[2])-1] != '/'):
             results.warning("Setting '%s': The path component of the URL ('%s') is expected by Django to end in a slash ('/'). Engage will add a slash" %
                             (setting_name, sr[2]))
-        logging.debug("Setting '%s' is a valid URL" % setting_name)
+        logger.debug("Setting '%s' is a valid URL" % setting_name)
     except:
         results.error("Invalid URL format for setting '%s': '%s'" %
                       (setting_name, setting_value))
@@ -155,27 +154,34 @@ def check_directory_tuple_setting(setting_name, setting_value,
                                     None, app_dir_path, results)
 
 
-def validate_installed_app(app_name, python_path_dir, known_apps, results):
-    if app_name in known_apps:
+def validate_installed_app(app_name, python_path_dir, known_apps,
+                           python_path_subdirectory, results):
+    try:
+        import_module(app_name)
         logger.debug("Application '%s' Ok." % app_name)
-        return
-    app_dirs = app_name.split(".")
-    current_dir = python_path_dir
-    for dirname in app_dirs:
-        current_dir = os.path.join(current_dir, dirname)
-        if not os.path.exists(current_dir):
+    except:
+        (exc_typ, exc_val, exc_tb) = sys.exc_info()
+        if exc_typ==ImportError and str(exc_val)==("No module named %s" % app_name):
+            logger.debug("Unable to find module named %s" % app_name)
             if app_name in PACKAGES_FOR_KNOWN_APPS:
                 pkgs = PACKAGES_FOR_KNOWN_APPS[app_name]
                 if len(pkgs)>1:
-                    results.warning("Django Application '%s' not found. Perhaps you wanted to include one of the following packages in your requirements.txt file: %s" %
+                    results.error("Django Application '%s' not found. Perhaps you wanted to include one of the following packages in your requirements.txt file: %s" %
                                     (app_name, pkgs))
                 else:
-                    results.warning("Django Application '%s' not found. Perhaps you wanted to include the '%s' package in your requirements.txt file" %
-                                    (app_name, pkgs[0]))
+                    results.error("Django Application '%s' not found. Perhaps you wanted to include the '%s' package in your requirements.txt file" %
+                                  (app_name, pkgs[0]))
             else:
-                results.warning("Django Application '%s' not found within the application's directory tree. This may be OK if you have an asssociated entry in your requirements.txt file. If it is supposed to be a python package included with you application, check the value you provided for the settings module." % app_name)
-            return
-    logger.debug("Application '%s' Ok." % app_name)
+                if len(python_path_subdirectory)>0:
+                    pp_msg = "PYTHONPATH was set to the directory <install_path_to_app>/%s" % python_path_subdirectory
+                else:
+                    pp_msg = "PYTHONPATH was set to the parent of your project root directory."
+                results.error("Django Application '%s' not found. If it is from an external package, add the associated package to the requirements.txt file. If the application is supposed to be a python module included with you project tree, check the value you provided for the settings module. %s" %
+                                  (app_name, pp_msg))
+        else: # another exception occurred in import
+            logger.exception("Exception when trying to import app %s" % app_name)
+            results.warning("Django Application '%s' was found, but an error occurred when trying to import the application: %s(%s)" %
+                          (app_name, exc_typ.__name__, str(exc_val)))
 
 def validate_post_install_commands(app_name, settings_module, results):
     """Validate that each named command in ENGAGE_DJANGO_POSTINSTALL_COMMANDS
@@ -209,28 +215,32 @@ def validate_fixture_file(fixture_name, installed_apps, fixture_dirs,
         return
     # next, look in the fixtures directory under all the installed apps
     for app_module in installed_apps:
-        if app_module in known_apps:
-            continue # we can't check the django apps fixtures
-        fixture_dir = os.path.join(app_module_name_to_dir(python_path_dir,
-                                                          app_module, False),
-                                   "fixtures")
-        for filename in filenames:
-            fixture_path = os.path.join(fixture_dir, filename)
-            if os.path.exists(fixture_path):
-                logging.debug("Found fixture %s at %s" % (fixture_name, fixture_path))
-                return
+        try:
+            m = import_module(app_name)
+            fixture_dir = os.path.join(os.path.dirname(m.__file__),
+                                       "fixtures")
+        except: # as a fallback, look under our python path
+            fixture_dir = os.path.join(app_module_name_to_dir(python_path_dir,
+                                                              app_module, False),
+                                       "fixtures")
+        if os.path.isdir(fixture_dir):
+            for filename in filenames:
+                fixture_path = os.path.join(fixture_dir, filename)
+                if os.path.exists(fixture_path):
+                    logger.debug("Found fixture %s at %s" % (fixture_name, fixture_path))
+                    return
     # next, look under all the specified fixture directories
     for dirname in fixture_dirs:
         for filename in filenames:
             fixture_path = os.path.join(dirname, filename)
             if os.path.exists(fixture_path):
-                logging.debug("Found fixture %s at %s" % (fixture_name, fixture_path))
+                logger.debug("Found fixture %s at %s" % (fixture_name, fixture_path))
                 return
     # finally, look relative to settings_file_directory
     for filename in filenames:
         fixture_path = os.path.join(settings_file_directory, filename)
         if os.path.exists(fixture_path):
-            logging.debug("Found fixture %s at %s" % (fixture_name, fixture_path))
+            logger.debug("Found fixture %s at %s" % (fixture_name, fixture_path))
             return
     # if we got here, we didn't find the fixture file anywhere
     results.error("Unable to find fixture %s" % fixture_name)
@@ -242,7 +252,7 @@ def validate_migration_apps(migration_apps, installed_apps, results):
         for installed_app in installed_apps:
             # look for an exact match or a suffix
             if (app == installed_app) or (installed_app[len(installed_app)-len(app)-1:] == ("." + app)):
-                logging.debug("Found migration app %s" % app)
+                logger.debug("Found migration app %s" % app)
                 found = True
                 break
         if not found:
@@ -264,10 +274,10 @@ def get_user_required_packages(app_dir_path):
     """
     requirements_file = os.path.join(app_dir_path, "requirements.txt")
     if os.path.exists(requirements_file):
-        logging.debug("Found requirements.txt file")
+        logger.debug("Found requirements.txt file")
         return parse_requirements.parse(requirements_file)
     else:
-        logging.debug("Did not find requirements.txt file")
+        logger.debug("Did not find requirements.txt file")
         return []
 
 def _get_subdir_component(parent_dir, subdir):
@@ -330,9 +340,10 @@ def validate_settings(app_dir_path, django_settings_module, django_config=None,
         settings_module = import_module(deployed_settings_module)
     except:
         (exc_type, exc_value, exc_traceback) = sys.exc_info()
-        logger.error("Exception in settings file import: %s" %
-                     traceback.format_exception(exc_type, exc_value, exc_traceback))
-        raise SettingsImportError("Error in settings import: %s(%s)" % (exc_type, exc_value))
+        logger.exception("Exception in settings file import: %s(%s)" %
+                         (exc_type.__name__, str(exc_value)))
+        raise SettingsImportError("Error in settings import: %s(%s)" %
+                                  (exc_type.__name__, str(exc_value)))
         
     # Check that the settings controlled by engage weren't overridden by app.
     # If any are overridden, we treat them as warnings.
@@ -371,9 +382,10 @@ def validate_settings(app_dir_path, django_settings_module, django_config=None,
     if hasattr(settings_module, "INSTALLED_APPS"):
         installed_apps = []
         packages = PREINSTALLED_PACKAGES + get_user_required_packages(app_dir_path)
-        known_apps = set(extend_app_list(get_apps_for_packages(packages)))
+        known_apps = set(get_apps_for_packages(packages))
         for app_name in settings_module.INSTALLED_APPS:
-            validate_installed_app(app_name, python_path_dir, known_apps, results)
+            validate_installed_app(app_name, python_path_dir, known_apps,
+                                   results.python_path_subdirectory, results)
             installed_apps.append(app_name)
     else:
         installed_apps = []
@@ -455,11 +467,14 @@ def validate_settings(app_dir_path, django_settings_module, django_config=None,
     return results # all done
 
         
-def run_installed_tests_as_subprocess(app_dir_path, django_settings_module, python_exe_path=sys.executable, use_logger=None):
+def run_installed_tests_as_subprocess(app_dir_path, django_settings_module,
+                                      python_exe_path=sys.executable,
+                                      use_logger=None, read_config_file=True):
     """Run the installed mode tests as a separate subprocess, using the installed app's environment.
     A results file is used to communcate back the results of the validation. After the run is complete,
     an instance of ParsedJsonResults is returned
     """
+    logger.info(">> Validating application settings")
     app_dir_path = os.path.abspath(os.path.expanduser(app_dir_path))
     if not os.path.exists(app_dir_path):
         raise TestSetupError("Django settings file validator: application directory '%s' does not exist" % app_dir_path)
@@ -469,6 +484,7 @@ def run_installed_tests_as_subprocess(app_dir_path, django_settings_module, pyth
                              (django_settings_module, app_dir_path))
     if use_logger == None:
         use_logger = logger
+    use_logger.debug("Setting PYTHONPATH to %s" % python_path)
     # we create a temporary file that will contain the results
     tf = tempfile.NamedTemporaryFile(delete=False)
     tf.write("null")
@@ -477,13 +493,13 @@ def run_installed_tests_as_subprocess(app_dir_path, django_settings_module, pyth
     try:
         program_and_args = [python_exe_path, script_file, "-v",
                             "validate-installed", app_dir_path, tf.name]
-        if hasattr(use_logger, "action"):
-            use_logger.action(' '.join(program_and_args))
-        else:
-            use_logger.info(' '.join(program_and_args))
+        if not read_config_file:
+            program_and_args.append(django_settings_module)
+        use_logger.debug(' '.join(program_and_args))
         env = {
             "PYTHONPATH": python_path,
-            "DJANGO_SETTINGS_MODULE": "%s" % django_settings_module
+            "DJANGO_SETTINGS_MODULE": "%s" % \
+               get_deployed_settings_module(django_settings_module)
         }
         subproc = subprocess.Popen(program_and_args,
                                    env=env, stdin=subprocess.PIPE,
@@ -493,7 +509,7 @@ def run_installed_tests_as_subprocess(app_dir_path, django_settings_module, pyth
                                                             subproc.pid))
         subproc.stdin.close()
         for line in subproc.stdout:
-            use_logger.debug("[%d] %s" % (subproc.pid, line))
+            use_logger.debug("[%d] %s" % (subproc.pid, line.rstrip()))
         subproc.wait()
         use_logger.debug("[%d] %s exited with return code %d" % (subproc.pid,
                                                                  program_and_args[0],
@@ -511,13 +527,20 @@ def run_installed_tests_as_subprocess(app_dir_path, django_settings_module, pyth
 class CommandManager(command.CommandManager):
     def __init__(self):
         super(CommandManager, self).__init__()
+        self.verbose = None
 
     def add_options(self, parser):
-        parser.add_option("-v", "--verbose", action="store_true", dest="verbose")
+        parser.add_option("-v", "--verbose", action="store_true", dest="verbose",
+                          default=False)
+        parser.add_option("--test-install-dir", dest="test_install_dir",
+                          default=None,
+                          help="Use the specified directory as a target for test installation, rather than a temporary directory")
+        parser.add_option("--package-cache-dir", dest="package_cache_dir",
+                          default=None,
+                          help="Directory to check for local copies of packages")
 
     def process_generic_options(self, options, args):
-        global verbose
-        verbose = options.verbose
+        self.verbose = options.verbose
 
 command_manager = CommandManager()
 
@@ -550,9 +573,55 @@ class PrepareCommandBase(Command):
         elif self.archive_file[-4:]==".tgz" or self.archive_file[-7:]==".tar.gz":
             self.archive_type = "tgz"
         else:
-            parser.error("Archive filename '%s' must end in either .zip (zip archive), .tgz, or .tar.gz (gzipped tar archive)" % self.archive_file)        
+            parser.error("Archive filename '%s' must end in either .zip (zip archive), .tgz, or .tar.gz (gzipped tar archive)" % self.archive_file)
 
-    def _validate_and_prepare_dir(self, app_dir_path, undo_ops):
+    def _validate_and_prepare_dir(self, app_dir_path, undo_ops, python_exe):
+        results = run_installed_tests_as_subprocess(app_dir_path,
+                                                    self.django_settings_module,
+                                                    python_exe_path=python_exe,
+                                                    read_config_file=False)
+        if len(results.error_messages)>0:
+            for error in results.error_messages:
+                logger.error("ERROR>> %s" % error)
+        if len(results.warning_messages)>0:
+            for warning in results.warning_messages:
+                logger.warning("WARNING>> %s" % warning)
+        results.print_final_status_message(logger)
+        if results.get_return_code() != SUCCESS_RC:
+            return results.get_return_code()
+        if results.product:
+            product = results.product
+        else:
+            product = "Custom Django Application"
+        if results.product_version:
+            product_version = results.product_version
+        else:
+            product_version = ""
+        config = DjangoConfig(product, product_version,
+                              self.django_settings_module,
+                              results.python_path_subdirectory, VERSION,
+                              installed_apps=results.installed_apps,
+                              fixtures=results.fixtures,
+                              migration_apps=results.migration_apps,
+                              components=results.components,
+                              post_install_commands=results.post_install_commands)
+        write_json(config.to_json(), os.path.join(app_dir_path, DJANGO_CFG_FILENAME))
+
+        # undo the changes we made
+        for (op_fun, args) in undo_ops:
+            op_fun(*args)
+        # delete all the .pyc files
+        find_files(app_dir_path, "\.pyc$", os.remove)
+        # (re)create the archive
+        if self.archive_type == "zip":
+            archive = ZipfileHandler(self.archive_file, "w")
+        else:
+            archive = TarfileHandler(self.archive_file, "w:gz")
+        archive.create_new_from_dir(app_dir_path)
+        archive.close()
+        return SUCCESS_RC
+        
+    def _validate_and_prepare_dir_old(self, app_dir_path, undo_ops):
         """Called by the subclass run() method after setting up the directory.
         """
         results = validate_settings(app_dir_path, self.django_settings_module)
@@ -624,15 +693,21 @@ class PrepareDirCommand(PrepareCommandBase):
         self.django_settings_module = args[1]
 
     def run(self):
-        tmpdir = tempfile.mkdtemp(suffix='-engage')
+        if self.options.test_install_dir:
+            install_dir = os.path.abspath(os.path.expanduser(self.options.test_install_dir))
+            os.makedirs(install_dir)
+        else:
+            install_dir = tempfile.mkdtemp(suffix='-engage')
         try:
-            app_dir_path = os.path.join(tmpdir, os.path.basename(self.django_project_directory))
-            shutil.copytree(self.django_project_directory, app_dir_path)
-            undo_ops = generate_settings_file(app_dir_path,
-                                              self.django_settings_module)
-            return self._validate_and_prepare_dir(app_dir_path, undo_ops)
+            (app_dir_path, undo_ops, python_exe) = \
+                simulated_install(self.django_project_directory,
+                                  install_dir, self.django_settings_module,
+                                  package_cache_dir=self.options.package_cache_dir)
+            return self._validate_and_prepare_dir(app_dir_path, undo_ops,
+                                                  python_exe)
         finally:
-            shutil.rmtree(tmpdir)
+            if not self.options.test_install_dir:
+                shutil.rmtree(install_dir)
 
 command_manager.register_command(PrepareDirCommand)
 
@@ -667,13 +742,21 @@ class PrepareArchiveCommand(PrepareCommandBase):
                          self.archive_file)
 
     def run(self):
-        tmpdir = tempfile.mkdtemp(suffix='-engage')
+        if self.options.test_install_dir:
+            install_dir = os.path.abspath(os.path.expanduser(self.options.test_install_dir))
+            os.makedirs(install_dir)
+        else:
+            install_dir = tempfile.mkdtemp(suffix='-engage')
         try:
-            (app_dir_path, undo_ops) = simulated_install(self.archive_file, tmpdir,
-                                                         self.django_settings_module)
-            return self._validate_and_prepare_dir(app_dir_path, undo_ops)
+            (app_dir_path, undo_ops, python_exe) = \
+                simulated_install(self.archive_file, install_dir,
+                                  self.django_settings_module,
+                                  package_cache_dir=self.options.package_cache_dir)
+            return self._validate_and_prepare_dir(app_dir_path, undo_ops,
+                                                  python_exe)
         finally:
-            shutil.rmtree(tmpdir)
+            if not self.options.test_install_dir:
+                shutil.rmtree(install_dir)
 
 command_manager.register_command(PrepareArchiveCommand)
 
@@ -699,7 +782,7 @@ class ValidateSafeCommand(Command):
     def run(self):
         (common_dir, django_config) = \
             run_safe_validations_on_archive(self.django_archive_file)
-        logger.info("All safe mode checks passed.\n")
+        logger.info(">> All safe mode checks passed.")
         return SUCCESS_RC
 
 command_manager.register_command(ValidateSafeCommand)    
@@ -720,25 +803,32 @@ class ValidateFullCommand(Command):
     LONG_DESC = _vf_long_desc
     def __init__(self, parser, args, options):
         super(ValidateFullCommand, self).__init__(parser, args, options, 1)
-        self.django_archive_file = args[0]
-        if not os.path.exists(self.django_archive_file):
-            parser.error("Django archive file %s does not exist" % self.django_archive_file)
+        self.archive_file = args[0]
+        if not os.path.exists(self.archive_file):
+            parser.error("Django archive file %s does not exist" % self.archive_file)
             
     def run(self):
-        with create_handler(self.django_archive_file) as handler:
+        with create_handler(self.archive_file) as handler:
             (common_dir, django_config) = run_safe_validations(handler)
-            tmpdir = tempfile.mkdtemp(suffix="-engage")
-            try:
-                app_dir_path = os.path.join(tmpdir, common_dir)
-                logger.debug("Extracting application to '%s'" % app_dir_path)
-                handler.extract(tmpdir)
-                generate_settings_file(app_dir_path, django_config.django_settings_module)
-                results = validate_settings(app_dir_path, django_config.django_settings_module,
-                                            django_config)
-                results.print_final_status_message(logger)
-                return results.get_return_code()
-            finally:
-                shutil.rmtree(tmpdir)
+        if self.options.test_install_dir:
+            install_dir = os.path.abspath(os.path.expanduser(self.options.test_install_dir))
+            os.makedirs(install_dir)            
+        else:
+            install_dir = tempfile.mkdtemp(suffix="-engage")
+        try:
+            (app_dir_path, undo_ops, python_exe) = \
+                simulated_install(self.archive_file, install_dir,
+                                  django_config.django_settings_module,
+                                  package_cache_dir=self.options.package_cache_dir)
+            results = run_installed_tests_as_subprocess(app_dir_path,
+                                                        django_config.django_settings_module,
+                                                        python_exe_path=python_exe,
+                                                        read_config_file=True)
+            results.print_final_status_message(logger)
+            return results.get_return_code()
+        finally:
+            if self.options.test_install_dir:
+                shutil.rmtree(install_dir)
 
 command_manager.register_command(ValidateFullCommand)
 
@@ -758,22 +848,38 @@ class ValidateInstalledCommand(Command):
     SHORT_DESC = "Perform all validations on an installed django application"
     LONG_DESC = _vi_long_desc
     def __init__(self, parser, args, options):
-        super(ValidateInstalledCommand, self).__init__(parser, args, options, 2)
+        super(ValidateInstalledCommand, self).__init__(parser, args, options, None)
+        num_args = len(args)
+        # We have a 'hidden' third argument which is the django settings module.
+        # If specified, we will not try to read the django config file and
+        # use that instead. This is only for being called as a subprocess.
+        if num_args!=2 and num_args!=3:
+            parser.error("Expecting 2 arguments for %s command." %
+                         ValidateInstalledCommand.NAME)
         self.app_dir_path = args[0]
         self.results_file = args[1]
+        if num_args==3:
+            self.django_settings_module = args[2]
+        else:
+            self.django_settings_module = None
         if not os.path.isdir(self.app_dir_path):
             parser.error("Django application directory %s does not exist or is not a directory" % self.app_dir_path)
-    
+        
     def get_results_file(self):
         return self.results_file
 
     def run(self):
-        config_file = os.path.join(self.app_dir_path, DJANGO_CFG_FILENAME)
-        if not os.path.exists(config_file):
-            raise FileMissingError("Missing configuration file %s" % config_file)
-        with open(config_file, "rb") as fileobj:
-            django_config = django_config_from_json(fileobj.read(), COMPATIBLE_PACKAGER_VERSION)        
-        results = validate_settings(self.app_dir_path, django_config.django_settings_module,
+        if self.django_settings_module:
+            ds_mod = self.django_settings_module
+            django_config = None
+        else:
+            config_file = os.path.join(self.app_dir_path, DJANGO_CFG_FILENAME)
+            if not os.path.exists(config_file):
+                raise FileMissingError("Missing configuration file %s" % config_file)
+            with open(config_file, "rb") as fileobj:
+                django_config = django_config_from_json(fileobj.read(), COMPATIBLE_PACKAGER_VERSION)
+            ds_mod = django_config.django_settings_module
+        results = validate_settings(self.app_dir_path, ds_mod,
                                     django_config)
         write_json(results.to_json(), self.results_file)
         results.print_final_status_message(logger)
