@@ -14,7 +14,7 @@ def main(argv):
         root_logger = logging.getLogger()
         handler = logging.StreamHandler()
         root_logger.addHandler(handler)
-        if verbose:
+        if command_manager.verbose:
             root_logger.setLevel(logging.DEBUG)
             handler.setLevel(logging.DEBUG)
         else:
@@ -26,10 +26,10 @@ def main(argv):
         (exc_type, exc_value, exc_traceback) = sys.exc_info()
         traceback.print_exception(exc_type, exc_value, None, file=sys.stderr)
         if command.get_results_file():
-            write_json(create_results_json(ARCHIVE_VALIDATION_ERROR_RC, command.get_name(),
-                                           error_messages=[exc_value.__repr__()],
-                                           exception=traceback.format_exception(exc_type, exc_value, exc_traceback)),
-                        command.get_results_file())
+            write_json(FatalErrorResults(ARCHIVE_VALIDATION_ERROR_RC,
+                                         exc_type, exc_value,
+                                         exc_traceback).to_json(),
+                       command.get_results_file())
         return ARCHIVE_VALIDATION_ERROR_RC
     except SystemExit, c:
         sys.exit(c)
@@ -37,9 +37,10 @@ def main(argv):
         (exc_type, exc_value, exc_traceback) = sys.exc_info()
         traceback.print_exception(exc_type, exc_value, exc_traceback, file=sys.stderr)
         if command.get_results_file():
-            write_json(create_results_json(UNEXPECTED_EXC_RC, command.get_name(),
-                                             exception=traceback.format_exception(exc_type, exc_value, exc_traceback)),
-                        command.get_results_file())
+            write_json(FatalErrorResults(UNEXPECTED_EXC_RC,
+                                         exc_type, exc_value,
+                                         exc_traceback).to_json(),
+                       command.get_results_file())
         return UNEXPECTED_EXC_RC 
 
 
